@@ -69,6 +69,8 @@ class SSAValue(ABC):
     uses: Set[Use] = field(init=False, default_factory=set, repr=False)
     """All uses of the value."""
 
+    name: Optional[str] = field(init=False, default=None)
+
     @staticmethod
     def get(arg: SSAValue | Operation) -> SSAValue:
         """Get a new SSAValue from either a SSAValue, or an operation with a single result."""
@@ -106,7 +108,7 @@ class SSAValue(ABC):
         if safe_erase and len(self.uses) != 0:
             raise Exception(
                 "Attempting to delete SSA value that still has uses.")
-        self.replace_by(ErasedSSAValue(self.typ))
+        self.replace_by(ErasedSSAValue(self.typ, self))
 
 
 @dataclass
@@ -121,7 +123,7 @@ class OpResult(SSAValue):
 
     def __repr__(self) -> str:
         return f"OpResult(typ={repr(self.typ)}, num_uses={repr(len(self.uses))}" + \
-            f", op_name={repr(self.op.name)}, result_index={repr(self.result_index)}"
+            f", op_name={repr(self.op.name)}, result_index={repr(self.result_index)}, name={repr(self.name)})"
 
     def __eq__(self, other):
         return self is other
@@ -163,6 +165,8 @@ class ErasedSSAValue(SSAValue):
     This is used during transformations when a SSA variable is destroyed but still used.
     """
 
+    old_value: SSAValue
+
     def __hash__(self):
         return hash(id(self))
 
@@ -245,6 +249,15 @@ class Operation:
     parent: Optional[Block] = field(default=None, repr=False)
     """The block containing this operation."""
 
+    def parent_block(self) -> Optional[Block]:
+        return self.parent
+
+    def parent_op(self) -> Optional[Operation]:
+        return self.parent.parent.parent if self.parent and self.parent.parent else None
+
+    def parent_region(self) -> Optional[Region]:
+        return self.parent.parent if self.parent else None
+
     @property
     def operands(self) -> FrozenList[SSAValue]:
         return self._operands
@@ -419,6 +432,15 @@ class Block:
     parent: Optional[Region] = field(default=None, init=False, repr=False)
     """Parent region containing the block."""
 
+    def parent_op(self) -> Optional[Operation]:
+        return self.parent.parent if self.parent else None
+
+    def parent_region(self) -> Optional[Region]:
+        return self.parent
+
+    def parent_block(self) -> Optional[Block]:
+        return self.parent.parent.parent if self.parent and self.parent.parent else None
+
     def __repr__(self) -> str:
         return f"Block(_args={repr(self._args)}, num_ops={len(self.ops)})"
 
@@ -520,8 +542,10 @@ class Block:
         for op in ops:
             self.add_op(op)
 
-    def insert_op(self, ops: Union[Operation, List[Operation]],
-                  index: int) -> None:
+    def insert_op(self,
+                  ops: Union[Operation, List[Operation]],
+                  index: int,
+                  name: Optional[str] = None) -> None:
         """
         Insert one or multiple operations at a given index in the block.
         The operations should not be attached to another block.
@@ -532,6 +556,10 @@ class Block:
             )
         if not isinstance(ops, list):
             ops = [ops]
+        if name:
+            for curr_op in ops:
+                for res in curr_op.results:
+                    res.name = name
         for op in ops:
             self._attach_op(op)
         self.ops = self.ops[:index] + ops + self.ops[index:]
@@ -624,6 +652,15 @@ class Region:
     parent: Optional[Operation] = field(default=None, init=False, repr=False)
     """Operation containing the region."""
 
+    def parent_block(self) -> Optional[Block]:
+        return self.parent.parent if self.parent else None
+
+    def parent_op(self) -> Optional[Operation]:
+        return self.parent
+
+    def parent_region(self) -> Optional[Region]:
+        return self.parent.parent.parent if self.parent and self.parent.parent else None
+
     def __repr__(self) -> str:
         return f"Region(num_blocks={len(self.blocks)})"
 
